@@ -58,6 +58,12 @@ InitInput (int argc, char **argv)
     KdInitInput ();
 }
 
+void
+CloseInput (void)
+{
+    KdCloseInput ();
+}
+
 #ifdef DDXBEFORERESET
 void
 ddxBeforeReset (void)
