@@ -42,7 +42,6 @@
 #include "scrnintstr.h"
 #include "pixmapstr.h"
 #include "extnsionst.h"
-#include "xf86drm.h"
 #include "xfixes.h"
 #include "dri2.h"
 #include "protocol-versions.h"
@@ -51,12 +50,15 @@
 #include "xf86Module.h"
 
 static ExtensionEntry	*dri2Extension;
+extern Bool DRI2ModuleSetup(void);
 
 static Bool
 validDrawable(ClientPtr client, XID drawable, Mask access_mode,
 	      DrawablePtr *pDrawable, int *status)
 {
-    *status = dixLookupDrawable(pDrawable, drawable, client, 0, access_mode);
+    *status = dixLookupDrawable(pDrawable, drawable, client,
+				M_DRAWABLE_WINDOW | M_DRAWABLE_PIXMAP,
+				access_mode);
     if (*status != Success) {
 	client->errorValue = drawable;
 	return FALSE;
@@ -91,7 +93,7 @@ ProcDRI2QueryVersion(ClientPtr client)
 
     WriteToClient(client, sizeof(xDRI2QueryVersionReply), &rep);
 
-    return client->noClientException;
+    return Success;
 }
 
 static int
@@ -129,7 +131,7 @@ ProcDRI2Connect(ClientPtr client)
     WriteToClient(client, rep.driverNameLength, driverName);
     WriteToClient(client, rep.deviceNameLength, deviceName);
 
-    return client->noClientException;
+    return Success;
 }
 
 static int
@@ -151,7 +153,19 @@ ProcDRI2Authenticate(ClientPtr client)
     rep.authenticated = DRI2Authenticate(pDraw->pScreen, stuff->magic);
     WriteToClient(client, sizeof(xDRI2AuthenticateReply), &rep);
 
-    return client->noClientException;
+    return Success;
+}
+
+static void
+DRI2InvalidateBuffersEvent(DrawablePtr pDraw, void *priv)
+{
+    xDRI2InvalidateBuffers event;
+    ClientPtr client = priv;
+
+    event.type = DRI2EventBase + DRI2_InvalidateBuffers;
+    event.drawable = pDraw->id;
+
+    WriteEventsToClient(client, 1, (xEvent *)&event);
 }
 
 static int
@@ -167,11 +181,12 @@ ProcDRI2CreateDrawable(ClientPtr client)
 		       &pDrawable, &status))
 	return status;
 
-    status = DRI2CreateDrawable(client, pDrawable, stuff->drawable);
+    status = DRI2CreateDrawable(client, pDrawable, stuff->drawable,
+				DRI2InvalidateBuffersEvent, client);
     if (status != Success)
 	return status;
 
-    return client->noClientException;
+    return Success;
 }
 
 static int
@@ -186,11 +201,11 @@ ProcDRI2DestroyDrawable(ClientPtr client)
 		       &pDrawable, &status))
 	return status;
 
-    return client->noClientException;
+    return Success;
 }
 
 
-static void
+static int
 send_buffers_reply(ClientPtr client, DrawablePtr pDrawable,
 		   DRI2BufferPtr *buffers, int count, int width, int height)
 {
@@ -198,6 +213,9 @@ send_buffers_reply(ClientPtr client, DrawablePtr pDrawable,
     int skip = 0;
     int i;
 
+    if (buffers == NULL)
+	    return BadAlloc;
+
     if (pDrawable->type == DRAWABLE_WINDOW) {
 	for (i = 0; i < count; i++) {
 	    /* Do not send the real front buffer of a window to the client.
@@ -234,6 +252,7 @@ send_buffers_reply(ClientPtr client, DrawablePtr pDrawable,
 	buffer.flags = buffers[i]->flags;
 	WriteToClient(client, sizeof(xDRI2Buffer), &buffer);
     }
+    return Success;
 }
 
 
@@ -252,16 +271,15 @@ ProcDRI2GetBuffers(ClientPtr client)
 	return status;
 
     if (DRI2ThrottleClient(client, pDrawable))
-	return client->noClientException;
+	return Success;
 
     attachments = (unsigned int *) &stuff[1];
     buffers = DRI2GetBuffers(pDrawable, &width, &height,
 			     attachments, stuff->count, &count);
 
 
-    send_buffers_reply(client, pDrawable, buffers, count, width, height);
+    return send_buffers_reply(client, pDrawable, buffers, count, width, height);
 
-    return client->noClientException;
 }
 
 static int
@@ -279,15 +297,13 @@ ProcDRI2GetBuffersWithFormat(ClientPtr client)
 	return status;
 
     if (DRI2ThrottleClient(client, pDrawable))
-	return client->noClientException;
+	return Success;
 
     attachments = (unsigned int *) &stuff[1];
     buffers = DRI2GetBuffersWithFormat(pDrawable, &width, &height,
 				       attachments, stuff->count, &count);
 
-    send_buffers_reply(client, pDrawable, buffers, count, width, height);
-
-    return client->noClientException;
+    return send_buffers_reply(client, pDrawable, buffers, count, width, height);
 }
 
 static int
@@ -324,7 +340,7 @@ ProcDRI2CopyRegion(ClientPtr client)
 
     WriteToClient(client, sizeof(xDRI2CopyRegionReply), &rep);
 
-    return client->noClientException;
+    return Success;
 }
 
 static void
@@ -342,21 +358,19 @@ vals_to_card64(CARD32 lo, CARD32 hi)
 
 static void
 DRI2SwapEvent(ClientPtr client, void *data, int type, CARD64 ust, CARD64 msc,
-	      CARD64 sbc)
+	      CARD32 sbc)
 {
-    xDRI2BufferSwapComplete event;
+    xDRI2BufferSwapComplete2 event;
     DrawablePtr pDrawable = data;
 
     event.type = DRI2EventBase + DRI2_BufferSwapComplete;
-    event.sequenceNumber = client->sequence;
     event.event_type = type;
     event.drawable = pDrawable->id;
     event.ust_hi = (CARD64)ust >> 32;
     event.ust_lo = ust & 0xffffffff;
     event.msc_hi = (CARD64)msc >> 32;
     event.msc_lo = msc & 0xffffffff;
-    event.sbc_hi = (CARD64)sbc >> 32;
-    event.sbc_lo = sbc & 0xffffffff;
+    event.sbc = sbc;
 
     WriteEventsToClient(client, 1, (xEvent *)&event);
 }
@@ -381,7 +395,7 @@ ProcDRI2SwapBuffers(ClientPtr client)
      * also orders swaps.
      */
     if (DRI2ThrottleClient(client, pDrawable))
-	return client->noClientException;
+	return Success;
 
     target_msc = vals_to_card64(stuff->target_msc_lo, stuff->target_msc_hi);
     divisor = vals_to_card64(stuff->divisor_lo, stuff->divisor_hi);
@@ -399,7 +413,7 @@ ProcDRI2SwapBuffers(ClientPtr client)
 
     WriteToClient(client, sizeof(xDRI2SwapBuffersReply), &rep);
 
-    return client->noClientException;
+    return Success;
 }
 
 static void
@@ -439,7 +453,7 @@ ProcDRI2GetMSC(ClientPtr client)
 
     WriteToClient(client, sizeof(xDRI2MSCReply), &rep);
 
-    return client->noClientException;
+    return Success;
 }
 
 static int
@@ -466,7 +480,7 @@ ProcDRI2WaitMSC(ClientPtr client)
     if (status != Success)
 	return status;
 
-    return client->noClientException;
+    return Success;
 }
 
 int
@@ -481,7 +495,7 @@ ProcDRI2WaitMSCReply(ClientPtr client, CARD64 ust, CARD64 msc, CARD64 sbc)
 
     WriteToClient(client, sizeof(xDRI2MSCReply), &rep);
 
-    return client->noClientException;
+    return Success;
 }
 
 static int
@@ -501,7 +515,7 @@ ProcDRI2SwapInterval(ClientPtr client)
 
     DRI2SwapInterval(pDrawable, stuff->interval);
 
-    return client->noClientException;
+    return Success;
 }
 
 static int
@@ -586,7 +600,7 @@ SProcDRI2Connect(ClientPtr client)
     rep.driverNameLength = 0;
     rep.deviceNameLength = 0;
 
-    return client->noClientException;
+    return Success;
 }
 
 static int
@@ -623,6 +637,8 @@ DRI2ExtensionInit(void)
 				 StandardMinorOpcode);
 
     DRI2EventBase = dri2Extension->eventBase;
+
+    DRI2ModuleSetup();
 }
 
 extern Bool noDRI2Extension;
