@@ -77,18 +77,13 @@
 #undef _POSIX_SOURCE
 #endif /* _POSIX_SOURCE */
 
-#if !defined(PATH_MAX)
-#if defined(MAXPATHLEN)
-#define PATH_MAX MAXPATHLEN
-#else
-#define PATH_MAX 1024
-#endif /* MAXPATHLEN */
-#endif /* !PATH_MAX */
-
 #if !defined(MAXHOSTNAMELEN)
 #define MAXHOSTNAMELEN 32
 #endif /* !MAXHOSTNAMELEN */
 
+/* For PATH_MAX */
+#include "misc.h"
+
 #include "Configint.h"
 #include "xf86tokens.h"
 
@@ -115,53 +110,6 @@ static int pushToken = LOCK_TOKEN;
 static int eol_seen = 0;		/* private state to handle comments */
 LexRec val;
 
-/* 
- * xf86strToUL --
- *
- *  A portable, but restricted, version of strtoul().  It only understands
- *  hex, octal, and decimal.  But it's good enough for our needs.
- */
-static unsigned int
-xf86strToUL (char *str)
-{
-	int base = 10;
-	char *p = str;
-	unsigned int tot = 0;
-
-	if (*p == '0')
-	{
-		p++;
-		if ((*p == 'x') || (*p == 'X'))
-		{
-			p++;
-			base = 16;
-		}
-		else
-			base = 8;
-	}
-	while (*p)
-	{
-		if ((*p >= '0') && (*p <= ((base == 8) ? '7' : '9')))
-		{
-			tot = tot * base + (*p - '0');
-		}
-		else if ((base == 16) && (*p >= 'a') && (*p <= 'f'))
-		{
-			tot = tot * base + 10 + (*p - 'a');
-		}
-		else if ((base == 16) && (*p >= 'A') && (*p <= 'F'))
-		{
-			tot = tot * base + 10 + (*p - 'A');
-		}
-		else
-		{
-			return tot;
-		}
-		p++;
-	}
-	return tot;
-}
-
 /*
  * xf86getNextLine --
  *
@@ -434,7 +382,7 @@ again:
 				configRBuf[i++] = c;
 			configPos--;		/* GJA -- one too far */
 			configRBuf[i] = '\0';
-			val.num = xf86strToUL (configRBuf);
+			val.num = strtoul (configRBuf, NULL, 0);
 			val.realnum = atof (configRBuf);
 			return NUMBER;
 		}
@@ -601,7 +549,6 @@ xf86pathIsSafe(const char *path)
  *    %P    projroot
  *    %C    sysconfdir
  *    %D    datadir
- *    %M    config file format version number
  *    %%    %
  */
 
@@ -626,11 +573,6 @@ xf86pathIsSafe(const char *path)
 #ifndef XCONFENV
 #define XCONFENV	"XORGCONFIG"
 #endif
-/* xorg.conf is based on XF86Config version 4.   If we ever break
-   compatibility of the xorg.conf syntax, we'll bump this version number. */
-#ifndef CONFIG_FILE_VERSION
-#define CONFIG_FILE_VERSION	4
-#endif
 
 #define BAIL_OUT		do {									\
 							free(result);				\
@@ -661,7 +603,6 @@ DoSubstitution(const char *template, const char *cmdline, const char *projroot,
 	int i, l;
 	static const char *env = NULL;
 	static char *hostname = NULL;
-	static char majorvers[3] = "";
 
 	if (!template)
 		return NULL;
@@ -762,13 +703,6 @@ DoSubstitution(const char *template, const char *cmdline, const char *projroot,
 			case 'D':
 				APPEND_STR(DATADIR);
 				break;
-			case 'M':
-				if (!majorvers[0]) {
-					snprintf(majorvers, sizeof(majorvers),
-						 "%d", CONFIG_FILE_VERSION);
-				}
-				APPEND_STR(majorvers);
-				break;
 			case '%':
 				result[l++] = '%';
 				CHECK_LENGTH;
@@ -819,6 +753,7 @@ OpenConfigFile(const char *path, const char *cmdline, const char *projroot,
 		}
 	}
 
+	free(pathcopy);
 	if (file) {
 		configFiles[numFiles].file = file;
 		configFiles[numFiles].path = strdup(filepath);
@@ -927,6 +862,7 @@ OpenConfigDir(const char *path, const char *cmdline, const char *projroot,
 		}
 	}
 
+	free(pathcopy);
 	return dirpath;
 }
 
@@ -1088,8 +1024,7 @@ void
 xf86setSection (char *section)
 {
 	free(configSection);
-	configSection = malloc(strlen (section) + 1);
-	strcpy (configSection, section);
+	configSection = strdup(section);
 }
 
 /* 
